/*	Hitachi HD44780 - kompatible LCD-Ansteuerung
*	Betrieben wir das Display in 4-Bit-Mode
*	(c) 2008 Tobias John (john.tobi@yahoo.de)
*	
*/

/* Ein Byte schreiben */
void lcd_write(unsigned char c) {
	unsigned char buff;
	buff = c >> 4;	// Die hochwertigen 4 Bits zwischenspeichern
	LCD_DATA &= 0x0F;	// D7-D4 lschen
	LCD_DATA |= (buff << 4);	// hochwertige 4 Bits auf LCD-Port
	LCD_STROBE;	// Daten bernehmen
	
	buff = c&0x0F;	// Niederwertigen 4 Bits zwischenspeichern
	LCD_DATA &= 0x0F; // D7-D4 lschen
	LCD_DATA |= (buff << 4);	// niederwertige Bits schreiben
	LCD_STROBE;	//Daten bernehmen
	DelayUs(40);
}

/* Lschen und Cursor-Home */
void lcd_clear(void) {
	LCD_RS = 0;
	lcd_write(0x1);   //-- Clear the Display
	DelayMs(3);
	lcd_write(0x02);  //-- Home the display
	DelayMs(3);
}

/* Zeichenkette senden */
void lcd_puts(const char * s) {
	LCD_RS = 1;	// write characters
	while(*s)
		lcd_write(*s++);
}

/* Cursor an bestimmte Position setzen */
void lcd_goto(unsigned char pos) {
	LCD_RS = 0;
	lcd_write(0x80+pos);           //-- sets the DDRAM Address
}

/* Cursor to Home-Position */
void lcd_home(void) {
 LCD_RS=0;                //-- write command
 lcd_write(0x02);         //-- Home Everything
 DelayMs(3);
}

/* Cursor an oder aus */
void lcd_cursor(char onoff) {
 LCD_RS=0;
 if(onoff==CURSOR_OFF)  {         //-   0000 1DCB
	lcd_write(0x0C);              //- D on/off= 1 C on/off=0 B on/off=0
 }
 else {
	lcd_write(0x0F);              //- D=1 C=1 B=1   => Cursor ON + Blink
 }
 DelayMs(2);
}

/* Initalisierungsroutine - muss am Anfang einmal ausgefhrt wrden */
void lcd_init(void) {
	T_LCD_RS=0;                    //-- Set TRIS for output on port
	T_LCD_EN=0;                    //   For RS and EN pins
	T_LCD_RW=0;
	T_LCD_DATA &=0x0F;             //-- Set Data (High-Nibbles) as output und lasse die Low-Nibbles wie vorher eingestellt

	LCD_RS = 0;	// write control bytes
	DelayMs(15);	// power on delay
	
	LCD_DATA &=0x0F;
	LCD_DATA |= 0x3;	// attention!

	LCD_STROBE;
	DelayMs(5);
	LCD_STROBE;
	DelayUs(100);
	LCD_STROBE;

	DelayMs(5);
	LCD_DATA &=0x0F;
	LCD_DATA = 0x2;	// set 4 bit mode
	LCD_STROBE;
	DelayUs(60);

	lcd_write(0x28);	// 4 bit mode, 1/16 duty, 5x8 font
	lcd_write(0x08);	// display off
	lcd_write(0x0F);	// display on, blink curson on
	lcd_write(0x06);	// entry mode

	DelayUs(100);
}


void lcd_var_zhl(unsigned char z) {
    unsigned char i,temp,Dez;
    bit k;                      // Merker ob vorne 0 ist
	LCD_RS = 1;
    k=0;
    Dez=100;                  // Dezimalstellen, z.B. bei 2 Stellen 10 eintragen
    while (Dez) {                 // Solange Dezimalstelle nicht 0 ist
		if(Dez==1)
			k=1;         // Um ein einzige 0 anzeigen knnen
		i=z/Dez;                // Wert der jeweilige Dezimalstelle bestimmen
		temp=i*Dez;
		z=z-temp;
		if(i||k) {                // Damit wir vorne keine Nullen stehen haben.
			lcd_write((char)i+0x30);
			k=1;                // es gab schon eine Zahl die nicht 0 ist
		}
		else {                    // wen 0 ist und vorne keine Zahl war, dann
			lcd_write(' ');  // Leerzeichen ausgeben
		}
		Dez=Dez/10;
	}
}
